/*=====================================
 * BUMP pour rts
 * (c) BoBo
 *=====================================
 */

 #include <allegro.h>
 #include "rtsdat.h"
 #include <math.h>

#define     RESX      200
#define     RESY      150

#define     LOOPX      64
#define     LOOPY     128

 extern   BITMAP   * dblbuff;
 extern   DATAFILE * datfile;


 static   BITMAP   * image;
 static   RGB      * pal;
 static   RGB        palk[256];

 static   int        CheminX[LOOPX];
 static   int        CheminY[LOOPY];

 static   int        posx[64];
 static   int        posy[64];

typedef     struct
            {
                int     x;
                int     y;
            }
            point;


/*=========================
 * PRIMITIVES
 *=========================
 */

static dr_bump( point LightPos, int posx, int posy )
{
    int     x, y;
    int     iDiffx;             // Indices de diffrence entre les normale
    int     iDiffy;             // de luminosit et du point
    point       LNormale;       // Normale du vecteur lumire
    point       NNormale;       // Normale du point courant

    for( y=1; y<RESY-1; y++ )
    {
        // Calcul de la normale du point lumineux en Y
        LNormale.y = (int)((y - LightPos.y)/1.5);
        for( x=1; x<RESX-1; x++ )
        {
            // Calcul de la normale du point courant en Y
            NNormale.y = image->line[y+1][x] - image->line[y-1][x];
            // Indice de diffrence entre normale lumire et normale courante en Y
            iDiffy = 127-abs( LNormale.y - NNormale.y );
            if ( iDiffy < 0 ) iDiffy = 0;
            // Calcul normale lumire en X
            LNormale.x =(int) ((x - LightPos.x)/1.5);
            // Calcul normale du point courant en X
            NNormale.x = image->line[y][x+1] - image->line[y][x-1];
            // Indice de diffrence en X
            iDiffx = 127-abs( LNormale.x - NNormale.x );
            if (iDiffx < 0 ) iDiffx = 0;
            // Couleur du point courant
            dblbuff->line[y+posy][x+posx] = 32 + ((iDiffx * iDiffy) >> 7);
        }
    }
}

/*=============================
 * PRINCIPALES
 *=============================
 */


prepare_bump()
{
    int     i;
    image = (BITMAP *)datfile[BBUMP_PCX].dat;
    pal   = (RGB *)datfile[BBUMP_PAL].dat;

    // Calcule le chemin de la louze !
    for( i=0; i<LOOPX; i++ ) CheminX[i] = (int) ( (sin( 2 * PI * i / LOOPX )+1)*(RESX>>1) );
    for( i=0; i<LOOPY; i++ ) CheminY[i] = (int) ( (sin( 2 * PI * i / LOOPY )+1)*(RESY>>1) );

    for( i=0; i<128 ; i++ ) palk[i+32] = pal[i];

    for ( i = 0 ; i < 32 ; i++ )
    {
       posx[i] = 100+((20*fcos(i<<19)) >> 16);
       posy[i] = 100+((20*fsin(i<<19)) >> 16);
    }

}



do_bump(int iter)
{
    int     xInc = 1;           // Incrment de dcalage
    int     yInc = 1;           // idem
    point   LPos;               // Position du centre de luminosit
    int     it = iter & 31;

    xInc = yInc = 3;

    xInc = iter & 63;
    yInc = iter & 127;
    LPos.x = CheminX[xInc];
    LPos.y = CheminY[yInc];

    dr_bump( LPos,posx[it],posy[it] );
    rect(dblbuff,posx[it],posy[it],posx[it]+200,posy[it]+150,159);
}


fix_bump()
{
    set_palette_range( palk, 32, 160,0 );
}











stop_bump()
{
   return(0);
}

