/*=======================================
 * CUBE 3D : pour rts
 * repompe a mort de l'example d'allegro
 *=======================================
 */

#include <allegro.h>
#include "rtsdat.h"


#define NUM_VERTICES       8     /* a cube has eight corners */
#define NUM_FACES          6     /* a cube has six faces */
#define MAX_SHAPES         8


extern BITMAP     * dblbuff;
extern DATAFILE   * datfile;

static BITMAP     * texture;
static PALLETE      pal;
static int          NUM_SHAPES = 4;

enum { 
   wireframe, 
   flat_shaded, 
   gouraud,
   textured, 
   lit_texture,
   last_mode
} render_mode = wireframe;

typedef struct VTX
{
   fixed x, y, z;
} VTX;


typedef struct QUAD              /* four vertices makes a quad */
{
   VTX *vtxlist;
   int v1, v2, v3, v4;
} QUAD;


typedef struct SHAPE             /* store position of a shape */
{
   fixed x, y, z;                /* x, y, z position */
   fixed rx, ry, rz;             /* rotations */
   fixed dz;                     /* speed of movement */
   fixed drx, dry, drz;          /* speed of rotation */
} SHAPE;


VTX points[] =                   /* a cube, centered on the origin */
{
   /* vertices of the cube */
   { -32 << 16, -32 << 16, -32 << 16 },
   { -32 << 16,  32 << 16, -32 << 16 },
   {  32 << 16,  32 << 16, -32 << 16 },
   {  32 << 16, -32 << 16, -32 << 16 },
   { -32 << 16, -32 << 16,  32 << 16 },
   { -32 << 16,  32 << 16,  32 << 16 },
   {  32 << 16,  32 << 16,  32 << 16 },
   {  32 << 16, -32 << 16,  32 << 16 },
};


QUAD faces[] =                   /* group the vertices into polygons */
{
   { points, 0, 3, 2, 1 },
   { points, 4, 5, 6, 7 },
   { points, 0, 1, 5, 4 },
   { points, 2, 3, 7, 6 },
   { points, 0, 4, 7, 3 },
   { points, 1, 2, 6, 5 }
};


SHAPE shapes[MAX_SHAPES];        /* a list of shapes */


/* somewhere to put translated vertices */
VTX output_points[NUM_VERTICES * MAX_SHAPES];
QUAD output_faces[NUM_FACES * MAX_SHAPES];



/*=================================
 * PRIMITIVES
 *=================================
 */

static void init_shapes()
{
   int c;

   for (c=0; c<NUM_SHAPES; c++) {
    //  shapes[c].x = (random() & 0xFFFFFF) - 0x800000;
    //  shapes[c].y = (random() & 0xFFFFFF) - 0x800000;
      shapes[c].z = itofix(255);
      shapes[c].rx = 0;
      shapes[c].ry = 0;
      shapes[c].rz = 0;
      shapes[c].drx =   8 << 16;
      shapes[c].dry =   4 << 16;
      shapes[c].drz =   3 << 16;
   }

   shapes[0].x = -64 << 16;
   shapes[1].x = +64 << 16;
   shapes[2].x = -64 << 16;
   shapes[3].x = +64 << 16;
   shapes[0].y = -64 << 16;
   shapes[1].y = +64 << 16;
   shapes[2].y = +64 << 16;
   shapes[3].y = -64 << 16;

   shapes[0].rx = 15 << 16;
   shapes[1].ry = 36 << 16;
   shapes[2].rz = 71 << 16;
   shapes[3].rx = 35 << 16;

   shapes[0].dz =  10 << 16;
   shapes[1].dz =  15 << 16;
   shapes[2].dz =   8 << 16;
   shapes[3].dz =  12 << 16;

}


/* update shape positions */
static void animate_shapes()
{
   int c;

   for (c=0; c<NUM_SHAPES; c++) {
      shapes[c].z += shapes[c].dz;

      if ((shapes[c].z > itofix(350)) ||
	  (shapes[c].z < itofix(192)))
	 shapes[c].dz = -shapes[c].dz;

      shapes[c].rx += shapes[c].drx;
      shapes[c].ry += shapes[c].dry;
      shapes[c].rz += shapes[c].drz;
   }
}



static void translate_shapes()
{
   int c, d;
   MATRIX matrix;
   VTX *outpoint = output_points;
   QUAD *outface = output_faces;

   for (c=0; c<NUM_SHAPES; c++) {
      /* build a transformation matrix */
      get_transformation_matrix(&matrix, itofix(1),
				shapes[c].rx, shapes[c].ry, shapes[c].rz,
				shapes[c].x, shapes[c].y, shapes[c].z);

      /* output the vertices */
      for (d=0; d<NUM_VERTICES; d++) {
	 apply_matrix(&matrix, points[d].x, points[d].y, points[d].z, &outpoint[d].x, &outpoint[d].y, &outpoint[d].z);
	 persp_project(outpoint[d].x, outpoint[d].y, outpoint[d].z, &outpoint[d].x, &outpoint[d].y);
      }

      /* output the faces */
      for (d=0; d<NUM_FACES; d++) {
	 outface[d] = faces[d];
	 outface[d].vtxlist = outpoint;
      }

      outpoint += NUM_VERTICES;
      outface += NUM_FACES;
   }
}


/* draw a line (for wireframe display) */
static void wire(BITMAP *b, VTX *v1, VTX *v2)
{
   int col = MID(1, 65 - fixtoi(v1->z+v2->z) / 32, 65);
   line(b, fixtoi(v1->x), fixtoi(v1->y), fixtoi(v2->x), fixtoi(v2->y), col);
}


/* draw a quad */
static void quad(BITMAP *b, VTX *v1, VTX *v2, VTX *v3, VTX *v4, int mode)
{
   int col;

   /* four vertices */
   V3D vtx1 = { v1->x, v1->y, v1->z, 0,      0,      0 };
   V3D vtx2 = { v2->x, v2->y, v2->z, 31<<16, 0,      0 };
   V3D vtx3 = { v3->x, v3->y, v3->z, 31<<16, 31<<16, 0 };
   V3D vtx4 = { v4->x, v4->y, v4->z, 0,      31<<16, 0 };

   /* cull backfaces */
   if (polygon_z_normal(&vtx1, &vtx2, &vtx3) < 0)
      return;

   /* set up the vertex color, differently for each rendering mode */
   switch (mode) {

      case POLYTYPE_FLAT:
	 col = MID(1, 65 - fixtoi(v1->z+v2->z) / 32, 65);
	 vtx1.c = vtx2.c = vtx3.c = vtx4.c = col;
	 break;

      case POLYTYPE_GCOL:
	 vtx1.c = 17;
	 vtx2.c = 1;
	 vtx3.c = 49;
	 vtx4.c = 65;
	 break;

      case POLYTYPE_ATEX_LIT:
      case POLYTYPE_PTEX_LIT:
	 vtx1.c = MID(0, 255 - fixtoi(v1->z) / 4, 255);
	 vtx2.c = MID(0, 255 - fixtoi(v2->z) / 4, 255);
	 vtx3.c = MID(0, 255 - fixtoi(v3->z) / 4, 255);
	 vtx4.c = MID(0, 255 - fixtoi(v4->z) / 4, 255);
	 break; 
   }

   /* draw the quad */
   quad3d(b, mode, texture, &vtx1, &vtx2, &vtx3, &vtx4);
}


/* callback for qsort() */
static int quad_cmp(const void *e1, const void *e2)
{
   QUAD *q1 = (QUAD *)e1;
   QUAD *q2 = (QUAD *)e2;

   fixed d1 = q1->vtxlist[q1->v1].z + q1->vtxlist[q1->v2].z +
	      q1->vtxlist[q1->v3].z + q1->vtxlist[q1->v4].z;

   fixed d2 = q2->vtxlist[q2->v1].z + q2->vtxlist[q2->v2].z +
	      q2->vtxlist[q2->v3].z + q2->vtxlist[q2->v4].z;

   return d2 - d1;
}


/* draw the shapes calculated by translate_shapes() */
static void draw_shapes(BITMAP *b)
{
   int c;
   QUAD *face = output_faces;
   VTX *v1, *v2, *v3, *v4;

   /* depth sort */
   qsort(output_faces, NUM_FACES * NUM_SHAPES, sizeof(QUAD), quad_cmp);

   for (c=0; c < NUM_FACES * NUM_SHAPES; c++) {
      /* find the vertices used by the face */
      v1 = face->vtxlist + face->v1;
      v2 = face->vtxlist + face->v2;
      v3 = face->vtxlist + face->v3;
      v4 = face->vtxlist + face->v4;

      /* draw the face */
      switch (render_mode) {

	 case wireframe:
	    wire(b, v1, v2);
	    wire(b, v2, v3);
	    wire(b, v3, v4);
	    wire(b, v4, v1);
	    break;

	 case flat_shaded:
	 default:
	    quad(b, v1, v2, v3, v4, POLYTYPE_FLAT);
	    break;

	 case gouraud:
	    quad(b, v1, v2, v3, v4, POLYTYPE_GCOL);
	    break;

	 case textured:
	    quad(b, v1, v2, v3, v4, POLYTYPE_ATEX);
	    break;

	 case lit_texture:
	    quad(b, v1, v2, v3, v4, POLYTYPE_ATEX_LIT);
	    break;
      }

      face++;
   }
}

/*=============================
 * PRINCIPALES
 *=============================
 */

Calc_Cube()
{
   int c;

   for (c=1; c<65; c++)
   {
      pal[c].r = 32 + ( c - 1 )/2; pal[c].g = 12 + ( c - 1 )/2 ; pal[c].b = 8;
   }
   color_map = malloc(sizeof(COLOR_MAP));
   create_light_table(color_map, pal, 0, 0, 0, NULL);
}

prepare_cube()
{
//   texture = create_bitmap(32, 32);
   init_shapes();

}

fix_cube()
{
   set_pallete_range(pal,1,65,0);
}


int fst_trans = 0;

do_cube(int iter, int mode )
{
   int c, w, h;

   render_mode = mode;
   animate_shapes();

   translate_shapes();
   draw_shapes(dblbuff);

}


stop_cube()
{
   free(color_map);
}

