/*================================================
 * Starfield pour rts
 * (c) Disk91 / TeKnO-BeDoUiNs /
 *================================================
 */

 #include <allegro.h>

 static V3D      stars[700];
 static int      NUM_STARS1 = 350;
 static int      NUM_STARS  = 10;
 extern BITMAP * dblbuff;
 static fixed    dx,dy,dz;
 static fixed    rx,ry,rz;
 fixed star_x[700], star_y[700];
 static int      matcour=0;
 static int      matsuiv=1;

typedef struct VECTOR
{
  fixed x, y, z;
} VECTOR;

typedef struct MOVE
{
   fixed rx,ry,rz;
   fixed tx,ty,tz;
} MOVE;


 #define Z_NEAR            24
 #define Z_FAR             1024
 #define XY_CUBE           512

/*====================================
 * PRIMITIVES
 *====================================
 */

static void move_stars (void)
{
  int i;

  for (i=0; i<NUM_STARS1; i++)
  {
    stars[i].x += dx;
    stars[i].y += dy;
    stars[i].z += dz;

    if (stars[i].x > itofix(XY_CUBE >> 1))
      stars[i].x = itofix(-(XY_CUBE >> 1));
    else if (stars[i].x < itofix(-(XY_CUBE >> 1)))
      stars[i].x = itofix(XY_CUBE >> 1);

    if (stars[i].y > itofix(XY_CUBE >> 1))
      stars[i].y = itofix(-(XY_CUBE >> 1));
    else if (stars[i].y < itofix(-(XY_CUBE >> 1)))
      stars[i].y = itofix(XY_CUBE >> 1);

    if (stars[i].z > itofix(Z_FAR))
      stars[i].z = itofix(Z_NEAR);
    else if (stars[i].z < itofix(Z_NEAR))
      stars[i].z = itofix(Z_FAR);
  }
}

static void draw_stars (void)
{
  int i, c;
  MATRIX m;
  VECTOR outs[NUM_STARS];
  int x,y;

  get_transformation_matrix (&m,itofix(1),rx, ry, rz, dx, dy, dz);
  for (i=0; i<NUM_STARS; i++)
  {
    apply_matrix (&m, stars[i].x, stars[i].y, stars[i].z,
			  &outs[i].x, &outs[i].y, &outs[i].z);
    persp_project (outs[i].x, outs[i].y, outs[i].z, &star_x[i], &star_y[i]);
    c = (fixtoi(outs[i].z) >> 8) + 92;
    x = fixtoi(star_x[i]);
    y = fixtoi(star_y[i]);
    putpixel (dblbuff, x  , y  , c);
    putpixel (dblbuff, x+1, y  , c);
    putpixel (dblbuff, x  , y+1, c);
    putpixel (dblbuff, x+1, y+1, c);

  }
}


/*--- transition ---*/

MOVE    transit[5] = { { 0, 0, 0,
                         0, 0, -25 << 16
                       },
                       { 512 << 16, 0, 0,
                         0, +10 << 16, -25 << 16
                       },
                       { 256 << 16, 512 << 16, 0,
                         -5 << 16, 0, -25 <<16
                       },
                       { 0, 256 << 16, 0,
                         0, -10 << 16, +25 << 16
                       },
                       { 0,  0, 256 << 16,
                         3 << 16, -5 << 16, 0
                       }
                     };





make_transition(int iter)
{
   int mat;


   mat = iter % 512;

   rx = ( transit[matcour].rx + (( transit[matsuiv].rx - transit[matcour].rx ) >> 9)*mat) & 255;
   ry = ( transit[matcour].ry + (( transit[matsuiv].ry - transit[matcour].ry ) >> 9)*mat) & 255;
   rz = ( transit[matcour].rz + (( transit[matsuiv].rz - transit[matcour].rz ) >> 9)*mat) & 255;
   dx = transit[matcour].tx + (( transit[matsuiv].tx - transit[matcour].tx ) >> 9)*mat;
   dy = transit[matcour].ty + (( transit[matsuiv].ty - transit[matcour].ty ) >> 9)*mat;
   dz = transit[matcour].tz + (( transit[matsuiv].tz - transit[matcour].tz ) >> 9)*mat;

   if ( mat == 511 )
   {
     matcour = matsuiv;
     matsuiv = ( matsuiv + 1 ) % 5;
   }
}




/*==============================
 * PRINCIPALES
 *==============================
 */

 fix_pal_stars()
 {
  PALETTE pal;

  pal[92].r =  63; pal[92].g = 40; pal[92].b = 20;
  pal[93].r =  43; pal[93].g = 30; pal[93].b = 10;
  pal[94].r =  35; pal[94].g = 20; pal[94].b = 8;
  pal[95].r =  23; pal[95].g = 13; pal[95].b = 5;
  pal[96].r =  20; pal[96].g =  10; pal[96].b = 2;
  pal[97].r =  10; pal[97].g =   5; pal[97].b = 1;

  set_palette_range(pal,92,97,0);
 }

 prepare_stars()
 {
  int i;

  for (i=0; i<NUM_STARS1; i++)
  {
    stars[i].x = itofix ((random() % XY_CUBE) - (XY_CUBE >> 1));
    stars[i].y = itofix ((random() % XY_CUBE) - (XY_CUBE >> 1));
    stars[i].z = itofix ((random() % (Z_FAR - Z_NEAR)) + Z_NEAR);
  }
  dx = itofix(0);
  dy = itofix(0);
  dz = itofix(-20);
  rx = itofix(0);
  ry = itofix(0);
  rz = itofix(0);
 }


 do_stars(int iteration)
 {
   if ( iteration < 240 )
      NUM_STARS++;

   make_transition(iteration);
   move_stars();
   draw_stars();
 }

 stop_stars()
 {
    return 0;
 }
